########################################################
##    ZPaybox - Ease the access to PayBox payment system
##    Paybox.py - Main object file
##
##  Copyright (C) 2001+  IngeniWeb SARL (http://www.ingeniweb.com)
##
##    This software is copyrighted by IngeniWeb SARL. 
##    This software is protected by LGPL licence.
##    See LICENCE.txt
##
##    See README.txt for documentation
##
##  FileName : $RCSfile: Paybox.py,v $
##  Revision : $Revision: 1.3 $ ($Date: 2002/06/17 13:08:36 $)
##  Author : $Author: odeckmyn $
##  Version tag name (if any) : $Name:  $
########################################################

from AccessControl import ClassSecurityInfo
from Globals import InitializeClass 
from Acquisition import Implicit
from Globals import Persistent
from AccessControl.Role import RoleManager
from OFS.SimpleItem import Item
from Globals import HTML, HTMLFile, DTMLFile, MessageDialog
from OFS.PropertyManager import PropertyManager

from Products.PythonScripts.standard import special_formats



import os
import httplib, urllib
import string


## Helper functions

def isoLang2payboxLang(lang):
    """isoLang2payboxLang(lang) -> string

       Returns paybox language code ('FRA' or 'DEU' or 'GBR') given iso lang code ('fr', 'be', etc...).
       If iso code is not known, return "GBR" code (english)."""
    lang=string.lower(lang)
    if lang in ('fr', 'be', 'ca'):
        return 'FRA'
    elif lang in ('de', ):
        return 'DEU'
    else:
        return 'GBR'

def isoCurrency2payboxCurrency(currency):
    """isoCurrency2payboxCurrency(currency) -> string

       Returns paybox currency code ('250' or '978') given iso currency code ('FRF', 'EUR' supported only).
       If iso code is not known, raises an exception."""
    currency=string.upper(currency)
    if currency=="FRF":
        return "250"
    elif currency=="EUR":
        return "978"
    elif currency=="USD":
        return "840"
    else:
        raise "Paybox does not support '%s' currency code" % currency

def _buildHTMLForm(server, cgi, token ):
    """Makes a call to the paybox.cgi script with all the parameters and return the needed HTML Button"""
    params=urllib.urlencode(token)
    h=httplib.HTTP(server)
    h.putrequest("POST", cgi)
    h.putheader("Content-length", "%d" % len(params) )
    h.putheader("Accept", "text/plain")
    h.putheader("Host", server)
    h.endheaders()
    h.send(params)
    reply, msg, hdrs = h.getreply()
    data = h.getfile().read() # get the raw HTML
    lines=string.split(data,"\n")
    result=[]
    for line in lines:
        if string.find(string.upper(line), "<INPUT")!=-1 or string.find(string.upper(line), 'FORM')!=-1:
            result.append(line)
    return string.join(result, "\n")

## Paybox main class


Paybox_addForm = DTMLFile('dtml/addForm', globals())

def Paybox_add(self, id, REQUEST = {}):
    """Add Paybox Product"""
    p = Paybox(id)
    self._setObject(id, p)
    if REQUEST.has_key('RESPONSE'):
        return self.manage_main(self, REQUEST)
    else:
        return getattr(self, id)

class Paybox(Implicit, Persistent, RoleManager, PropertyManager, Item):
	"""Paybox Product Main Class"""

	meta_type="Paybox"

	security=ClassSecurityInfo()

	_properties=(
            {'id':   'server',
	     'type': 'string',
	     'mode': 'w'},
            
	    {'id':   'cgi',
	     'type': 'string',
	     'mode': 'w'},

	    {'id':   'site',
	     'type': 'string',
	     'mode': 'w'},

	    {'id':   'rang',
	     'type': 'string',
	     'mode': 'w'},

	    {'id':   'url_canceled',
	     'type': 'string',
	     'mode': 'w'},

	    {'id':   'url_completed',
	     'type': 'string',
	     'mode': 'w'},


	    {'id':   'url_refused',
	     'type': 'string',
	     'mode': 'w'},

	)

	manage_options = PropertyManager.manage_options + Item.manage_options + RoleManager.manage_options + (
	    {'label': 'ReadMe / HowTo', 'action': 'manage_ReadMe'},
	   )
	

	def __init__(self, id, server='locahost', cgi='/cgi-bin/paybox.cgi', site="1999888", rang="99", 
	             url_canceled='', url_completed='', url_refused=''):
		"""Constructor"""
		self.id=id
		self.server=server
		self.cgi=cgi
		self.site=site
		self.rang=rang
		self.url_canceled=url_canceled
		self.url_completed=url_completed
		self.url_refused=url_refused


	def physical_location(self):
    		app=self.getPhysicalRoot()
    		p=getattr(app.Control_Panel.Products, "ZPaybox", None)
    		if p:
        		return p.home
    		raise '%s not found in Control_Panel/Products' % ("ZPaybox", )

	def _get_params(self):
		return {
                         'IBS_MODE'    : "1",
                         'IBS_SITE'    : str(self.site),
                         'IBS_RANG'    : str(self.rang),
                         'IBS_ANNULE'  : self.url_canceled,
                         'IBS_EFFECTUE': self.url_completed,
                         'IBS_REFUSE'  : self.url_refused,
                         'IBS_OUTPUT'  : "A",
                        }



	security.declarePublic('manage_ReadMe')
	def manage_ReadMe(self):
		"""ReadMe File"""
		readme=open(os.path.join(self.physical_location(),"README.txt")).read()
                readme=special_formats['structured-text'](readme)
		return self.manage_page_header()+self.manage_tabs()+readme+self.manage_page_footer()

		
	security.declarePublic('__call__')
	def __call__(self, email, amount, reference, lang='fr', currency='EUR', sub_frequency=0, sub_amount=0, sub_length=0, sub_day=0, sub_delay=0, button='PAYBOX'):
		"""This is the main function. It builds the HTML Form for paybox system. Will ask local 
		 cgi script to build the correct form.
                 email (string) : email of customer,
                 amount (float) : amount of product,
                 currency (string) : iso code of currency (FRF or EUR only)
                 reference (string) : unique product reference,
                 lang (string, optionnal) : language to use on Paybox pages. French is default.

                 IF AND ONLY IF sub_frequency IS != 0 : the sub_ variables are considered and mean the following things :
                 - sub_frequency : subscription frequency (in monthes)
                 - sub_ammount : subscription ammount (or 0 if the same as pdt_amount)
                 - sub_length : subscription length (or 0 if infinite) ie. nb of payings
                 - sub_day : subscription day (or 0 if the same as today)
                 - sub_delay : subscription delay (nb. of days to wait before starting subscription, or 0 to start now)

		See test() for example usage."""
            	# Handling multiple frequency parameters
		if sub_frequency:
                	reference = "%sIBS_2MONT%010dIBS_NBPAIE%02dIBS_FREQ%02dIBS_QUAND%02dIBS_DELAIS%03d" % (
                    		reference, int(sub_amount*100), sub_length, sub_frequency, sub_day, sub_delay)

            	amount=int(amount*100) # 123.45 -> 12345
            	token={
                	'IBS_TOTAL':str(amount),
                	'IBS_DEVISE': isoCurrency2payboxCurrency(currency),
                	'IBS_CMD': reference,
                	'IBS_PORTEUR':email,
                	'IBS_LANGUE':isoLang2payboxLang(lang),
			'IBS_BOUTPI':button,
                	"IBS_RETOUR": "IBS_TOTAL:M;IBS_CMD:R;IBS_AUTH:A;IBS_TRANSAC:T",
            	}
            	token.update(self._get_params())
            	return _buildHTMLForm(self.server, self.cgi, token)
		
		
InitializeClass(Paybox)		
